package io.data2viz.charts.event

import io.data2viz.charts.chart.Chart
import io.data2viz.charts.core.Datum
import io.data2viz.math.Percent
import kotlin.jvm.JvmName

/**
 * Interface to control a [Chart]: highlight or select elements, zoom the view or pan it.
 */
public interface ChartController<DOMAIN> {

    /**
     * Highlight the given DOMAIN objects.
     *
     * @param domains A [Collection] of DOMAIN objects to highlight on screen.
     */
    @Suppress("INAPPLICABLE_JVM_NAME")
    @JvmName("highlightDomains")
    public fun highlight(domains: Collection<DOMAIN>)

    /**
     * Highlight the given [Datum]s.
     *
     * @param data A [Collection] of [Datum] to highlight on screen.
     */
    @Suppress("INAPPLICABLE_JVM_NAME")
    @JvmName("highlightData")
    public fun highlight(data: Collection<Datum<DOMAIN>>)

    /**
     * Select the given DOMAIN objects.
     *
     * @param domains A [Collection] of DOMAIN objects to highlight on screen.
     */
    @Suppress("INAPPLICABLE_JVM_NAME")
    @JvmName("selectDomains")
    public fun select(domains: Collection<DOMAIN>)

    /**
     * Select the given [Datum]s.
     *
     * @param data A [Collection] of [Datum] to highlight on screen.
     */
    @Suppress("INAPPLICABLE_JVM_NAME")
    @JvmName("selectData")
    public fun select(data: Collection<Datum<DOMAIN>>)

    /**
     * Display the [Cursor] on a given DOMAIN object. Set to null to hide cursor.
     *
     * @param domain A [DOMAIN]? object to place the cursor on.
     */
    public fun setCursorFor(domain: DOMAIN?)

    /**
     * Display the [Cursor] on a given Datum. Set to null to hide cursor.
     *
     * @param datum A [Datum] to place the cursor on.
     */
    public fun setCursorFor(datum: Datum<DOMAIN>?)

    /**
     * Zoom the "viewport" of the [Chart] by a certain percentage in X or Y direction, knowing the zoom origin.
     * For example zoom(50%, 50%, 110%, 110%) will zoom by a ratio 1.1 in X and Y directions around the middle of the
     * charting zone.
     *
     * @param zoomOriginX The origin position of the zoom in [Percent] along the X-axis.
     * @param zoomOriginY The origin position of the zoom in [Percent] along the Y-axis.
     * @param zoomFactorX The zooming ratio in X in [Percent].
     * @param zoomFactorY The zooming ratio in Y in [Percent].
     */
    public fun zoom(zoomOriginX: Percent, zoomOriginY: Percent, zoomFactorX: Percent, zoomFactorY: Percent)

    /**
     * Move the "viewport" of the [Chart] by a certain percentage in X or Y direction.
     * For example pan(-10%, 20%) will move the charting zone 10% to the left and 20% up.
     *
     * @param panX The percentage of panning in [Percent] along the X-axis.
     * @param panY The percentage of panning in [Percent] along the Y-axis.
     */
    public fun pan(panX: Percent, panY: Percent)

    /**
     * Reset the view, setting the pan and zoom to their origin values.
     */
    public fun viewReset()
}
